#include "StdAfx.h"
#include "Tutorial.h"

using namespace optix;

static float rand_range(float min, float max)
{
	return min + (max - min) * (float) rand() / (float) RAND_MAX;
}

Tutorial::~Tutorial(void)
{
}

// From SampleScene
void   Tutorial::initScene( InitialCameraData& camera_data )
{
	// ptxt@C̃pX𐶐
	_ptx_path = std::string(sutilSamplesPtxDir()) + "/tutorial0.ptx";
	std::cout << _ptx_path << std::endl;

	// ReLXg̐ݒ
	// C2
	_context->setRayTypeCount( 2 );
	// Gg[|Cg1ӏ
	_context->setEntryPointCount( 1 );
	// X^bNTCY8000
	_context->setStackSize( 8000 );

	// ep[^̐ݒ
	// ő̐[100
	_context["max_depth"]->setInt(100);
	// ̃C̎ނ0
	_context["radiance_ray_type"]->setUint(0);
	// ẽC̎ނ1
	_context["shadow_ray_type"]->setUint(1);
	// t[0
	_context["frame_number"]->setUint( 0u );
	// V[̃CvVݒ
	_context["scene_epsilon"]->setFloat( 1.e-3f );
	// JbgIt̒lݒ
	_context["importance_cutoff"]->setFloat( 0.01f );
	// ̐Fݒ
	_context["ambient_light_color"]->setFloat( 0.31f, 0.33f, 0.28f );

	// o̓obt@̍쐬Ɛݒ
	_context["output_buffer"]->set( createOutputBuffer(RT_FORMAT_UNSIGNED_BYTE4, _width, _height) );

	// CvO
	std::string camera_name;
	//  camera_name = "env_camera";
	// sz[J̍쐬
	camera_name = "pinhole_camera";

	// CvO̍쐬
	Program ray_gen_program = _context->createProgramFromPTXFile( _ptx_path, camera_name );
	// ReLXgɃCvOݒ
	_context->setRayGenerationProgram( 0, ray_gen_program );

	// Oƃ~XvO̍쐬
	Program exception_program = _context->createProgramFromPTXFile( _ptx_path, "exception" );
	// OvO̐ݒ
	_context->setExceptionProgram( 0, exception_program );
	// Oꍇ̐Fݒ
	_context["bad_color"]->setFloat( 0.0f, 1.0f, 0.0f );

	// ~XvOiȂꍇ͊}bṽeNX`gpj
	std::string miss_name;
	//  miss_name = "envmap_miss";
	// vO
	miss_name = "miss";
	// ~XvO̍쐬
	_context->setMissProgram( 0, _context->createProgramFromPTXFile( _ptx_path, miss_name ) );
	// }bvɃeNX`ݒ
	const float3 default_color = make_float3(1.0f, 1.0f, 1.0f);
	_context["envmap"]->setTextureSampler( loadTexture( _context, texpath("CedarCity.hdr"), default_color) );
	// wiF̐ݒ
	_context["bg_color"]->setFloat( make_float3( 0.34f, 0.55f, 0.85f ) );

	// 
	BasicLight lights[] = 
	{
		// ʒu, F, eLXg邩ǂ
		{ make_float3( -5.0f, 60.0f, -16.0f ), make_float3( 1.0f, 1.0f, 1.0f ), 1 }
	};

	// ̃obt@̍쐬
	Buffer light_buffer = _context->createBuffer(RT_BUFFER_INPUT);
	// [Uw̃tH[}bggp
	light_buffer->setFormat(RT_FORMAT_USER);
	// vf̃TCY
	light_buffer->setElementSize(sizeof(BasicLight));
	// ̐H
	light_buffer->setSize( sizeof(lights)/sizeof(lights[0]) );
	// obt@̃}bvɌRs[H
	memcpy(light_buffer->map(), lights, sizeof(lights));
	// obt@̃A}bvH
	light_buffer->unmap();

	// ReLXgɌݒ
	_context["lights"]->set(light_buffer);

	// J̐ݒ
	camera_data = InitialCameraData( make_float3( 7.0f, 9.2f, -6.0f ), // _ʒu
		make_float3( 0.0f, 4.0f,  0.0f ), // _
		make_float3( 0.0f, 1.0f,  0.0f ), // xNg
		60.0f );                          // p(fovy)

	// ReLXg̃J͑S0ɐݒH
	_context["eye"]->setFloat( make_float3( 0.0f, 0.0f, 0.0f ) );
	_context["U"]->setFloat( make_float3( 0.0f, 0.0f, 0.0f ) );
	_context["V"]->setFloat( make_float3( 0.0f, 0.0f, 0.0f ) );
	_context["W"]->setFloat( make_float3( 0.0f, 0.0f, 0.0f ) );

	// 3D̃\bhmCYobt@Bfloat^1`lŁA[0.0, 1.0]
	srand(0); // 񓯂ɂȂ悤0ݒ

	// 64x64x64̃mCYobt@𐶐
	int tex_width  = 64;
	int tex_height = 64;
	int tex_depth  = 64;
	Buffer noiseBuffer = _context->createBuffer(RT_BUFFER_INPUT, RT_FORMAT_FLOAT, tex_width, tex_height, tex_depth);
	// mCYobt@̃}bv
	float *tex_data = (float *) noiseBuffer->map();

	// [0, 1]̒lƂ闐𐶐
	for (int i = tex_width * tex_height * tex_depth;  i > 0; i--) {
		// 1`l3DmCYi[0.0, 1.0]j
		*tex_data++ = rand_range(0.0f, 1.0f);
	}
	// mCYobt@̃A}bv
	noiseBuffer->unmap(); 


	// mCYeNX`̃Tv̍쐬
	TextureSampler noiseSampler = _context->createTextureSampler();

	// eNX`̃p[^̐ݒ
	noiseSampler->setWrapMode(0, RT_WRAP_REPEAT);
	noiseSampler->setWrapMode(1, RT_WRAP_REPEAT);
	noiseSampler->setFilteringModes(RT_FILTER_LINEAR, RT_FILTER_LINEAR, RT_FILTER_NONE);
	noiseSampler->setIndexingMode(RT_TEXTURE_INDEX_NORMALIZED_COORDINATES);
	noiseSampler->setReadMode(RT_TEXTURE_READ_NORMALIZED_FLOAT);
	noiseSampler->setMaxAnisotropy(1.0f);
	noiseSampler->setMipLevelCount(1);
	noiseSampler->setArraySize(1);
	noiseSampler->setBuffer(0, 0, noiseBuffer);

	// ReLXgɃmCYeNX`ݒ
	_context["noise_texture"]->setTextureSampler(noiseSampler);

	// V[Kw̐
	createGeometry();

	// s邽߂̏
	_context->validate();
	_context->compile();
}

void   Tutorial::trace( const RayGenCameraData& camera_data )
{
	// CJf[^ReLXgɃp[^ݒ
	_context["eye"]->setFloat( camera_data.eye );
	_context["U"]->setFloat( camera_data.U );
	_context["V"]->setFloat( camera_data.V );
	_context["W"]->setFloat( camera_data.W );

	// ReLXg̏o̓obt@擾
	Buffer buffer = _context["output_buffer"]->getBuffer();
	RTsize buffer_width, buffer_height;
	// obt@̃TCY擾
	buffer->getSize( buffer_width, buffer_height );

	// g[X̎s
	_context->launch( 0, static_cast<unsigned int>(buffer_width),
		static_cast<unsigned int>(buffer_height) );
}

void   Tutorial::doResize( unsigned int width, unsigned int height )
{
	// output buffer handled in SampleScene::resize
}

Buffer Tutorial::getOutputBuffer()
{
	// ReLXg̏o̓obt@Ԃ
	return _context["output_buffer"]->getBuffer();
}

std::string Tutorial::texpath( const std::string& base )
{
	return texture_path + "/" + base;
}

void Tutorial::createGeometry()
{
	// {bNX֌W̃vOPTXt@C̃pX𐶐
	std::string box_ptx( ptxpath( "tutorial", "box.cu" ) );
	// {bNX̋EvZvO̐
	Program box_bounds = _context->createProgramFromPTXFile( box_ptx, "box_bounds" );
	// {bNX̌vZvO̐
	Program box_intersect = _context->createProgramFromPTXFile( box_ptx, "box_intersect" );

	// {bNX̐
	// WIg̍쐬
	Geometry box = _context->createGeometry();
	// v~eBuH̐ݒ
	box->setPrimitiveCount( 1u );
	// oEfBO{bNXvZvO̐ݒ
	box->setBoundingBoxProgram( box_bounds );
	// vZvO̐ݒ
	box->setIntersectionProgram( box_intersect );
	// {bNX̍ŏW
	box["boxmin"]->setFloat( -2.0f, 0.0f, -2.0f );
	// {bNX̍őW
	box["boxmax"]->setFloat(  2.0f, 7.0f,  2.0f );

	// chull̐͏ȗ

	// ̃WIg̍쐬
	// 𐶐PTXt@C̃pX𐶐
	std::string pgram_ptx( ptxpath( "tutorial", "parallelogram.cu" ) );
	// ̃WIg쐬
	Geometry parallelogram = _context->createGeometry();
	// v~eBuH1
	parallelogram->setPrimitiveCount( 1u );
	// oEfBO{bNXvZvO̐Ɛݒ
	parallelogram->setBoundingBoxProgram( _context->createProgramFromPTXFile( pgram_ptx, "bounds" ) );
	// vZvO̐Ɛݒ
	parallelogram->setIntersectionProgram( _context->createProgramFromPTXFile( pgram_ptx, "intersect" ) );
	// ̍ŏWH
	float3 anchor = make_float3( -64.0f, 0.01f, -64.0f );
	// X̒xNgH
	float3 v1 = make_float3( 128.0f, 0.0f, 0.0f );
	// Z̒xNgH
	float3 v2 = make_float3( 0.0f, 0.0f, 128.0f );
	// @xNg
	float3 normal = cross( v2, v1 );
	// @xNg̐K
	normal = normalize( normal );
	// @xNgƏ̍ŏW̓ρH
	float d = dot( normal, anchor );
	// H
	v1 *= 1.0f/dot( v1, v1 );
	v2 *= 1.0f/dot( v2, v2 );
	// ʂ̃p[^쐬
	float4 plane = make_float4( normal, d );
	// ̖ʂ̃p[^ݒ
	parallelogram["plane"]->setFloat( plane );
	// xNg̐ݒ
	parallelogram["v1"]->setFloat( v1 );
	parallelogram["v2"]->setFloat( v2 );
	// Jn_̐ݒ
	parallelogram["anchor"]->setFloat( anchor );

	// ގ̐ݒ
	std::string box_chname;
	// box_chname = "box_closest_hit_radiance";
	box_chname = "closest_hit_radiance3";
	// box_chname = "closest_hit_radiance2";
	// box_chname = "closest_hit_radiance1";
	// box_chname = "closest_hit_radiance0";

	// {bNX̃}eA쐬
	Material box_matl = _context->createMaterial();
	// PTXł߂qbg̃vO𐶐
	Program box_ch = _context->createProgramFromPTXFile( _ptx_path, box_chname );
	// {bNX̃}eAɍł߂qbg̃vOݒ
	box_matl->setClosestHitProgram( 0, box_ch );
	// e\ꍇ́Ał߂qbgł͂ȂASẴqbg̃vOƂĐݒ
	Program box_ah = _context->createProgramFromPTXFile( _ptx_path, "any_hit_shadow" );
	box_matl->setAnyHitProgram( 1, box_ah );

	// ambient̐ݒ
	box_matl["Ka"]->setFloat( 0.3f, 0.3f, 0.3f );
	// diffuse̐ݒ
	box_matl["Kd"]->setFloat( 0.6f, 0.7f, 0.8f );
	// specular̐ݒ
	box_matl["Ks"]->setFloat( 0.8f, 0.9f, 0.8f );
	// shininess̐ݒ
	box_matl["phong_exp"]->setFloat( 88 );
	// ˗H̐ݒ
	box_matl["reflectivity_n"]->setFloat( 0.2f, 0.2f, 0.2f );

	// ̃}eA̍쐬
	std::string floor_chname;
//	floor_chname = "floor_closest_hit_radiance";
//	floor_chname = "floor_closest_hit_radiance5";
//	floor_chname = "floor_closest_hit_radiance4";
	floor_chname = "closest_hit_radiance3";
//	floor_chname = "closest_hit_radiance2";
//	floor_chname = "closest_hit_radiance1";
//	floor_chname = "closest_hit_radiance0";

	// ̃}eA𐶐
	Material floor_matl = _context->createMaterial();
	// ̍ł߂qbg̃vO𐶐^ݒ
	Program floor_ch = _context->createProgramFromPTXFile( _ptx_path, floor_chname );
	floor_matl->setClosestHitProgram( 0, floor_ch );
	// e\ꍇ͑SẴqbg̃vO𐶐^ݒ
	Program floor_ah = _context->createProgramFromPTXFile( _ptx_path, "any_hit_shadow" );
	floor_matl->setAnyHitProgram( 1, floor_ah );

	// ̍ގݒ
	floor_matl["Ka"]->setFloat( 0.3f, 0.3f, 0.1f );
	floor_matl["Kd"]->setFloat( 194/255.f*.6f, 186/255.f*.6f, 151/255.f*.6f );
	floor_matl["Ks"]->setFloat( 0.4f, 0.4f, 0.4f );
	floor_matl["reflectivity"]->setFloat( 0.1f, 0.1f, 0.1f );
	floor_matl["reflectivity_n"]->setFloat( 0.05f, 0.05f, 0.05f );
	floor_matl["phong_exp"]->setFloat( 88 );
	floor_matl["tile_v0"]->setFloat( 0.25f, 0, .15f );
	floor_matl["tile_v1"]->setFloat( -.15f, 0, 0.25f );
	floor_matl["crack_color"]->setFloat( 0.1f, 0.1f, 0.1f );
	floor_matl["crack_width"]->setFloat( 0.02f );

	// OX̍ގݒ͏ȗ

	// ꂼ̃WIg̃CX^X𐶐
	std::vector<GeometryInstance> gis;
	gis.push_back( _context->createGeometryInstance( box, &box_matl, &box_matl+1 ) );
	gis.push_back( _context->createGeometryInstance( parallelogram, &floor_matl, &floor_matl+1 ) );
	// gis.push_back( _context->createGeometryInstance( chull, &glass_matl, &glass_matl+1 ) );

	// WIgO[v쐬đSĂzu
	GeometryGroup geometrygroup = _context->createGeometryGroup();
	geometrygroup->setChildCount( static_cast<unsigned int>(gis.size()) );
	geometrygroup->setChild( 0, gis[0] );
	geometrygroup->setChild( 1, gis[1] );
	// geometrygroup->setChild( 2, gis[2] );
	// ANZ[VH͎gȂH
	geometrygroup->setAcceleration( _context->createAcceleration("NoAccel","NoAccel") );

	// gbvIuWFNgɃWIgO[vݒ
	_context["top_object"]->set( geometrygroup );
	// gbvVhEiVhEvZWIgHjɃWIgO[vݒ
	_context["top_shadower"]->set( geometrygroup );
}

